@@ -57,14 +57,14 @@ bs_themer_ui <- function(opts, vals) {
     value <- vals[[id]]
     lbl <- HTML(opts$label)
     desc <- HTML(opts$desc)
-    text_input <- function(input_class = NULL) {
+    text_input <- function(input_class = NULL, type = "text", ...) {
       div(
         class = "form-row form-group",
         tags$label(lbl),
         tags$input(
-          type = "text", value = value, "data-id" = id,
+          type = type, value = value, "data-id" = id,
           class = "form-control form-control-sm bs-theme-value",
-          class = input_class
+          class = input_class, ...
         ),
         if (!is.null(desc)) div(class = "form-text small", desc)
       )
@@ -74,6 +74,7 @@ bs_themer_ui <- function(opts, vals) {
       color = text_input(input_class = "bs-theme-value-color text-monospace"),
       str = text_input(input_class = "bs-theme-value-str"),
       length = text_input(input_class = "bs-theme-value-length"),
+      number = text_input(input_class = "bs-theme-value-str", type = "number", step = opts$step),
       bool = tagList(
         div(
           class = "form-check",
@@ -136,8 +137,18 @@ bs_themer_ui <- function(opts, vals) {
     ),
 
     div(id = "bsthemerContainer",
-      class = "card shadow", style = css(width = "18rem", max_height = "80vh", z_index = 1000),
-      style = css(position = "fixed", top = "1rem", right = "1rem", height = "auto"),
+      class = "card shadow",
+      style = css(
+        # The bootstrap-colorpicker plugin sets a z-index of 1060 on
+        # it's inputs, so the container needs a smaller index, than that
+        # https://github.com/rstudio/bslib/blob/e4da71f3/inst/lib/bs-colorpicker/css/bootstrap-colorpicker.css#L38
+        #
+        # It's also important that this z-index is higher than 1030 so it's
+        # overlaid on-top of fixed/sticky navbars
+        # https://github.com/rstudio/bslib/blob/e4da71f3/inst/lib/bs/scss/_variables.scss#L697-L701
+        z_index = 1059, width = "18rem", max_height = "80vh",
+        position = "fixed", top = "1rem", right = "1rem", height = "auto"
+      ),
 
       div(id = "bsthemerHeader",
         class = "move-grabber", "data-target" = "#bsthemerContainer",
@@ -195,16 +206,14 @@ themer_css_dependency <- function(theme) {
 #'
 #' @section Limitations:
 #'
-#'   Currently, this utility only works with Bootstrap 4. We hope to add
-#'   Bootstrap 3 compatibility in the future. Also, the color picker currently
-#'   doesn't render correctly on IE11.
-#'
-#'   It also only works with Shiny apps and R Markdown apps that use the Shiny
-#'   runtime. It's not possible to perform real-time preview for static R
-#'   Markdown documents.
-#'
-#'   Note that only CSS generated with [bs_dependency_defer()] will be
-#'   instantly reflected in theme preview.
+#'   * Doesn't work with Bootstrap 3.
+#'   * Doesn't work with IE11.
+#'   * Only works inside Shiny apps and `runtime: shiny` R Markdown documents.
+#'     * Can't be used with static R Markdown documents.
+#'     * Can be used to some extent with `runtime: shiny_prerendered`, but only UI
+#'       rendered through a `context="server"` may update in real-time.
+#'   * Doesn't work with '3rd party' custom widgets that don't make use of
+#'     [bs_dependency_defer()] or [bs_current_theme()].
 #'
 #' @return nothing. These functions are called for their side-effects.
 #'
@@ -277,8 +286,8 @@ bs_themer <- function(gfonts = TRUE, gfonts_update = FALSE) {
     stop(call. = FALSE, "`bslib::bs_themer()` must be called from within a ",
          "top-level Shiny server function, not a Shiny module server function")
   }
-  if (!is_available("shiny", "1.5.0.9003")) {
-    stop(call. = FALSE, "`bslib::bs_themer()` requires shiny v1.5.0.9003 or higher")
+  if (!is_available("shiny", "1.6.0")) {
+    stop(call. = FALSE, "`bslib::bs_themer()` requires shiny v1.6.0 or higher")
   }
   theme <- get_current_theme()
   if (!is_bs_theme(theme)) {
@@ -303,12 +312,15 @@ bs_themer <- function(gfonts = TRUE, gfonts_update = FALSE) {
   themer_opts <- opts_metadata()
   themer_vars <- unlist(unname(lapply(themer_opts, names)))
   sass_vars <- setdiff(themer_vars, "bootswatch")
-  themer_vals <- as.list(bs_get_variables(theme, sass_vars))
+  themer_vals <- as.list(get_themer_vals(theme, sass_vars))
   themer_vals$bootswatch <- bootswatch
   shiny::insertUI("body", where = "beforeEnd", ui = bs_themer_ui(themer_opts, themer_vals))
 
   input <- session$input
 
+  # We emit different 'code' for runtime:shiny in Rmd
+  isRmd <- is_shiny_runtime()
+
   # When the bootswatch theme changes, update the themer's state to reflect
   # the new variable defaults. Note that we also update the "input theme",
   # and effectively throw out any other theming changes made (i.e., start from a new theme)
@@ -316,7 +328,8 @@ bs_themer <- function(gfonts = TRUE, gfonts_update = FALSE) {
   # consequence of a new bootswatch value
   shiny::observeEvent(input$bs_theme_bootswatch, {
     theme <<- set_current_theme(
-      theme, list(bootswatch = input$bs_theme_bootswatch), session
+      theme, list(bootswatch = input$bs_theme_bootswatch),
+      session, rmd = isRmd
     )
     vals <- as.list(bs_get_variables(theme, sass_vars))
     session$sendCustomMessage("bs-themer-bootswatch", list(values = vals))
@@ -343,11 +356,11 @@ bs_themer <- function(gfonts = TRUE, gfonts_update = FALSE) {
       return()
     }
 
-    # Remember, theme at this point has been updated to reflect the current Bootswatch theme
-    changed_vals <- as.list(diff_css_values(
-      vals[sass_vars],
-      bs_get_variables(theme, names(vals[sass_vars]))
-    ))
+    # Remember, theme at this point has been updated to reflect the current Bootswatch theme,
+    # so re-query Sass values from the (possibly updated) theme, then filter down to meaningful
+    # differences
+    theme_vals <- get_themer_vals(theme, names(vals[sass_vars]))
+    changed_vals <- as.list(diff_css_values(vals[sass_vars], theme_vals))
 
     if (!identical(bootswatch, input$bs_theme_bootswatch)) {
       changed_vals$bootswatch <- input$bs_theme_bootswatch
@@ -363,24 +376,69 @@ bs_themer <- function(gfonts = TRUE, gfonts_update = FALSE) {
     changed_vals <- rename2(
       changed_vals,
       "font-family-base" = "base_font", "font-family-monospace" = "code_font",
-      "headings-font-family" = "heading_font"
+      "headings-font-family" = "heading_font",
+      "font-size-base" = "font_scale"
     )
 
+    if (length(changed_vals$font_scale)) {
+      changed_vals$font_scale <- as.numeric(changed_vals$font_scale)
+    }
+
     if (isTRUE(gfonts)) {
       for (var in c("base_font", "code_font", "heading_font")) {
         changed_vals[[var]] <- insert_font_google_call(changed_vals[[var]], gfont_info)
       }
     }
 
-    set_current_theme(theme, changed_vals, session)
+    set_current_theme(theme, changed_vals, session, rmd = isRmd)
   })
 }
 
 
-set_current_theme <- function(theme, changed_vals, session) {
-  message("--------------------")
-  code <- rlang::expr(bs_theme_update(theme, !!!changed_vals))
-  print(code)
+get_themer_vals <- function(theme, vars) {
+  vals <- bs_get_variables(theme, vars)
+  if (!grepl("rem$", vals[["font-size-base"]])) {
+    stop("font-size-base must have a CSS unit length type of rem", call. = FALSE)
+  }
+  vals[["font-size-base"]] <- sub("rem$", "", vals[["font-size-base"]])
+  vals
+}
+
+set_current_theme <- function(theme, changed_vals, session, rmd = FALSE) {
+  shiny::insertUI("body", ui = spinner_overlay(), immediate = TRUE, session = session)
+  on.exit(shiny::removeUI("body > #spinner_overlay"), add = TRUE)
+
+  # Construct the code/yaml to display to the user
+  if (isTRUE(rmd)) {
+    display_vals <- lapply(changed_vals, function(x) {
+      if (is.numeric(x)) {
+        return(x)
+      }
+      if (rlang::is_call(x)) {
+        str <- paste0(deparse(x, width.cutoff = 500L), collapse = "")
+        return(paste("!expr", str))
+      }
+      # To avoid yaml parse errors with values that contain # or ",
+      # first escape ", then in quote the value
+      paste0('"', gsub('"', '\\"', x, fixed = TRUE), '"')
+    })
+    message("\n####  Update your Rmd output format's theme:  ####")
+    cat(paste0(
+      "    theme:\n",
+      paste0(
+        collapse = "\n", "      ", names(display_vals), ": ", display_vals
+      ),
+      "\n"
+    ))
+  } else {
+    message("\n####  Update your bs_theme() R code with:  #####")
+    print(rlang::expr(bs_theme_update(theme, !!!changed_vals)))
+  }
+
+  # Color contrast warnings are more annoying then they are useful inside the theming widget
+  opts <- options(bslib.color_contrast_warnings = FALSE)
+  on.exit(options(opts), add = TRUE)
+
   # the actual code that we evaluate should not have quoted expressions
   changed_vals[] <- lapply(changed_vals, eval_val)
   code <- rlang::expr(bs_theme_update(theme, !!!changed_vals))
@@ -405,6 +463,28 @@ set_current_theme <- function(theme, changed_vals, session) {
   invisible(theme)
 }
 
+spinner_overlay <- function() {
+  tagList(
+    tags$style(
+      "@supports ((-webkit-backdrop-filter:blur(4px)) or (backdrop-filter:blur(4px))) {
+        #spinner_overlay{ -webkit-backdrop-filter:blur(4px); backdrop-filter:blur(4px); background-color:rgba(255,255,255,.05);}
+      }"
+    ),
+    div(
+      id = "spinner_overlay",
+      style = "position:absolute; top:0; left:0; min-height:100vh; width:100%; background-color:rgba(255,255,255,.8); z-index:100000",
+      class = "d-flex flex-column justify-content-center align-items-center",
+      div(
+        class = "spinner-border",
+        style = "width:5rem; height:5rem; color: rgba(0,0,0,0.8);",
+        role = "status",
+        span(class = "sr-only", "Refreshing stylesheets...")
+      ),
+      span(class = "lead mt-1", style = "color: rgba(0,0,0,0.8);", "Refreshing stylesheets...")
+    )
+  )
+}
+
 eval_val <- function(x) {
   if (is.call(x)) return(eval(x))
   if (!is.list(x)) return(x)
@@ -416,7 +496,10 @@ insert_font_google_call <- function(val, gfont_info) {
   if (!is_string(val)) return(NULL)
   if (!nzchar(val)) return(NULL)
   fams <- strsplit(as.character(val), ",")[[1]]
-  fams <- vapply(fams, unquote_font_family, character(1), USE.NAMES = FALSE)
+  fams <- vapply(
+    fams, function(x) gsub("^\\s*['\"]?", "", gsub("['\"]?\\s*$", "", x)),
+    character(1), USE.NAMES = FALSE
+  )
   fams <- fams[nzchar(fams)]
   is_a_gfont <- tolower(fams) %in% tolower(gfont_info$family)
   if (length(fams) == 1) {
@@ -426,7 +509,7 @@ insert_font_google_call <- function(val, gfont_info) {
   for (i in which(is_a_gfont)) {
     fams[[i]] <- call("font_google", fams[[i]])
   }
-  unname(fams)
+  rlang::expr(font_collection(!!!unname(fams)))
 }
 
 
@@ -508,7 +591,7 @@ bs_get_variables <- function(theme, varnames) {
   css <- sass_partial(
     cssvars,
     # Add declarations to the current theme
-    bs_bundle(theme, sass_layer(declarations = sassvars)),
+    bs_bundle(theme, sass_layer(mixins = sassvars)),
   )
 
   # Search the output for the block of properties we just generated, using the
@@ -554,7 +637,7 @@ diff_css_values <- function(a, b) {
   stopifnot(all(!is.na(a)))
   stopifnot(identical(names(a), names(b)))
   stopifnot(is.list(a))
-  stopifnot(is.character(b))
+  if(!is.character(b))browser()
 
   a_char <- vapply(a, function(x) {
     if (is.null(x) || isTRUE(is.na(x))) {
